// cell.h
#ifndef __ode__Cell__
#define __ode__Cell__

#include <vector>
#include "RungeKutta4.h"

using std::vector;
class Cell : public RungeKutta4 
{
public:
    Cell(){};
    ~Cell(){};
    void setInitialValues(double v_init, vector<double>& y);
    void setPulse(double ton, double toff, double current);
    void rhs(double t, vector<double> y, vector<double>& yt);
private:
    static constexpr double Dm = 60.0;  // (micrometer) diameter of the cell
    static constexpr double Lm = 70.0;  // (micrometer) length of the cell

    static constexpr double Eleak = -65.0;   // (mV)
    static constexpr double ENa = 50.0;   // (mV)
    static constexpr double EK = -100.0;   // (mV)
    static constexpr double ECa = 50.0;   // (mV)

    static constexpr double dGleak = 0.002;// (mS/cm^2)
    static constexpr double dGNa   = 0.05; //0.53;   // (mS/cm^2)
    static constexpr double dGK    = 0.15; // 0.13;  // (mS/cm^2)
    static constexpr double dGTCa  = 0.03;  //  (mS/cm^2)

    static constexpr double vtraub = -55.0; //  -55.0 mV
    static constexpr double fac = 10;  // conversion factor (mS/cm^2) -> (pS/micrometer^2)

    int ndim;
//     double v,vinit;
    double Iinj;   // 2000.0 (pA)
    double Ton;    //   20 ms
    double Toff;   //   40 ms

    double Am;     //  surface area
    double Cm;
    double InvCm;  //  1/Cm;

    double Gleak;
    double GNa;
    double GK;
    double GTCa;

private:
    void get_Na_m(double v, double* a, double* b);
    void get_Na_h(double v, double* a, double* b);
    void get_K_n(double v, double* a, double* b);
    void get_TCa_m(double v, double* tau, double* inf);
    void get_TCa_h(double v, double* tau, double* inf);
};

#endif /* defined(__ode__Cell__) */
