// release.cpp
// Stochastic neurotransmitter release
// Ralf Schneggenburger & Erwin Neher
// Nature 406, 889-893, 2000.
#include <iostream>
#include <fstream>
#include <vector>
#include <cmath>
#include "UniRand.h"
int main(int argc, const char * argv[]){
    const double ttime = 4.0;
    const double dt=0.02;
    const double peak = 20.0;
    const double hw = 1.0;
    const int iter = 100000;

    const double kon  =  9.0e-2; // microM^{-1} ms^{-1}
    const double koff =  9.5;    // ms^{-1}
    const double beta =  0.25;   // cooperativity
    const double gamma = 6.0;    // 6.0d0  ! ms^{-1}
    const double epsilon = 0.3;  // ms^{-1}
    const int initialState = 1;

    std::cout << iter << " iterations" << std::endl;

    int npts = (int)(ttime/dt);
    int counter = 0;
    std::vector<double> ca(npts, 0.0);
    std::vector<int> sum(npts,0);

    UniRand urng; // Uniform random number generator

    // ca transient
    double d = 2.0*sqrt((1.0 - sqrt(0.5)))/hw;
    for(int i=0;i<(int)(2.0/(dt*d));i++){
        ca[i] = peak * pow(pow(d*dt*i - 1.0, 2.0)-1.0, 2.0);
    }

    double konh   = dt * kon;
    double koffh  = dt * koff;
    double gammah = dt * gamma;
    double epsilonh = dt*epsilon;
    double beta2 = beta*beta;
    double beta3 = beta*beta2;
    double beta4 = beta*beta3;

    for(int k=0;k<iter;k++){    // iteration loop
        int s  = initialState;  //initial state
        for(int i=0;i<npts;i++){
            double th;
            double r = urng.rand();                  // get a random number [0, 1]
            double alpha = konh * ca[i];  // get Ca level
            int snext = s;
            switch (s){
                case (1):
                    th = 5.0 * alpha;
                    if(r<th) snext = 2;
                    break;
                case (2):
                    th = 4.0 * alpha;
                    if(r<th) {
                        snext = 3;
                    }else{
                        th = th + koffh;
                        if(r<th) snext = 1;
                    }
                    break;
                case (3):
                    th = 3.0 * alpha;
                    if(r<th) {
                        snext = 4;
                    }else{
                        th = th + koffh * beta;
                        if(r<th) snext = 2;
                    }
                    break;
                case (4):
                    th = 2.0 * alpha;
                    if(r<th){
                        snext = 5;
                    }else{
                        th = th + koffh * beta2;
                        if(r<th) snext = 3;
                    }
                    break;
                case (5):
                    th = alpha;
                    if(r<th){
                        snext = 6;
                    }else{
                        th = th + koffh * beta3;
                        if(r<th) snext = 4 ;
                    }
                    break;
                case (6):
                    th = gammah;
                    if(r<th){
                        snext = 7;
                    }else{
                        th = th + koffh * beta4;
                        if(r<th) snext = 5;
                    }
                    break;
                case (7):
                    snext = 0;     //release
                    sum[i] += 1;  // release
                    counter++;
                    break;
                case (0):         //recovery from release
                    th = epsilonh;
                    if(r<th) snext = 1;
                    break;
                default:
                    ;
            } // end of switch
            s = snext;
        } //end of i-loop
    } // end of k-loop

    std::cout << "release probability = " << double(counter)/(double)iter << std::endl;;
    std::fstream fd("ax.txt", std::ios::out | std::ios::trunc);
    for(int i=0;i<npts;i++){
        fd << dt*i << " " << ca[i] << " " << (double)sum[i] << std::endl;
    }
    fd.close();
#if defined(_WIN32)
    system("..\\..\\tools\\gc ax.txt  &");
#else
    system("../../tools/gc ax.txt  &");
#endif
    return 0;
}
