!  ode.f90

program ode
    implicit none
    integer, parameter :: ndim = 2 ! number of equations
    double precision, parameter :: ttime = 10.0d0 ! total time
    double precision, parameter :: h = 0.1d0     ! step size

    double precision, dimension(ndim) :: y = (/1.0d0, 0.0d0/)

    integer :: npts, i
    double precision :: t

    npts = int(ttime/h)+1;
    
    open(unit = 11, file='ax.txt', form='formatted', access='stream', status='replace')
    do i = 1, npts
        t = h*i
        write(unit=11, fmt='(3E15.7)') t, y
        call advanceStep(t, y)
    end do
    close(unit=11)

    call system("../../tools/gc ax.txt ");

contains

    subroutine advanceStep(t, y)
        double precision, intent(in) :: t
        double precision, dimension(:), intent(inout) :: y 
        double precision, dimension(ndim) :: yn, k1, k2, k3, k4

        call rhs(t, y, k1)
        yn = y + h * k1/2.0d0
        
        call rhs(t+h/2.0d0, yn, k2)
        yn = y + h*k2/2.0d0
        
        call rhs(t+h/2.0d0, yn, k3)
        yn = y + h*k3

        call rhs(t+h, yn, k4);
        y = y + h*(k1 + 2.0d0*k2 + 2.0d0*k3 + k4)/6.0d0
    end subroutine advanceStep


    subroutine rhs (tt, y, dy)
        double precision :: tt
        double precision, dimension(:), intent(in):: y
        double precision, dimension(:), intent(out):: dy
        dy(1) =  y(2)
        dy(2) = -y(1)
    end subroutine rhs

end program ode