! channel.f90

program channel
    implicit none

    integer, parameter :: ndim = 9 ! number of equations
    double precision, parameter :: ttime = 50.0d0 ! total time
    double precision, parameter :: h = 0.01d0     ! step size
    integer, parameter :: npts = floor(ttime/h)+1;
    double precision :: vold  = -1000.0d0
    
    double precision :: a11, a12, a13, b11, b12, b13
    double precision :: a2, b2, a3, b3, a4, b4, a5, b5
    double precision :: v

    double precision :: t
    double precision, dimension(ndim) :: y
    integer :: i
 
    y(1) = 1.0d0
    y(2:ndim) = 0.0d0
    
    open(unit = 11, file='ax.txt', form='formatted', access='stream', status='replace')
    do i = 1, npts
        t = h*i
        write(unit=11, fmt='(6E15.7)') t, y(1), y(4), y(7:9)
        call advanceStep(t, y)
    end do
    close(unit=11)
    
    call system("../../tools/gc ax.txt ");

!   call execute_command_line("..\\..\\tools\\gc ax.txt ");  

contains

    subroutine advanceStep(tt, y)
        double precision, intent(in) :: tt
        double precision, dimension(:), intent(inout) :: y 
        double precision, dimension(ndim) :: yn, k1, k2, k3, k4
        
        call rhs(tt, y, k1)
        yn = y + h * k1/2.0d0
        
        call rhs(tt+h/2.0d0, yn, k2)
        yn = y + h*k2/2.0d0
        
        call rhs(tt+h/2.0d0, yn, k3)
        yn = y + h*k3

        call rhs(tt+h, yn, k4);
        y = y + h*(k1 + 2.0d0*k2 + 2.0d0*k3 + k4)/6.0d0
    end subroutine advanceStep

    subroutine rhs (tt, y, dy)
        double precision, intent(in) :: tt
        double precision, dimension(:), intent(in):: y
        double precision, dimension(:), intent(out):: dy
    
    
        if(tt<20.0d0 .or. tt>= 30.0d0)then
            v = -100.0d0
        else
            v = 0.0d0
        end if
        if (v .ne. vold ) then
            a11 = 3.802d0/(0.1027d0*exp(-v/17.0d0) + 0.20d0*exp(-v/150.0d0))
            a12 = 3.802d0/(0.1027d0*exp(-v/15.0d0) + 0.23d0*exp(-v/150.0d0))
            a13 = 3.802d0/(0.1027d0*exp(-v/12.0d0) + 0.25d0*exp(-v/150.0d0))

            b11 = 0.1917d0*exp(-v/20.3d0)
            b12 = 0.20d0*exp(-(v-5.0d0)/20.3d0)
            b13 = 0.22d0*exp(-(v-10.0d0)/20.3d0)

            a3 = (3.7933d-7)*exp(-v/7.7d0)
            b3 = 0.0084d0 + 0.00002d0*v

            a2 = 9.178d0*exp(v/29.68d0)
            b2 = (a13*a2*a3)/(b13*b3)

            a4 = a2/100d0
            b4 = a3

            a5 = a2/(9.5d4)
            b5 = a3/50.0d0          
            vold = v
        end if

        dy(1) = -(a11 + b3)*y(1) + b11*y(2) + a3*y(5)
        dy(2) = -(b11 + b3 + a12)*y(2) + a11*y(1) + b12*y(3) + a3*y(6)
        dy(3) = -(b12 + b3 + a13)*y(3) + a12*y(2) + b13*y(4) + a3*y(7)
        dy(4) = -(b13 + a2)*y(4) + a13*y(3) + b2*y(7)
        dy(5) = -(a11 + a3)*y(5) + b11*y(6) + b3*y(1)
        dy(6) = -(b11 + a3 + a12)*y(6) + a11*y(5) + b12*y(7) + b3*y(2)
        dy(7) = -(b12 + a3 + b2 + a4)*y(7) + a12*y(6) + b3*y(3) + a2*y(4) + b4*y(8)
        dy(8) = -(b4 + a5)*y(8) + a4*y(7) + b5*y(9)
        dy(9) = -b5*y(9) + a5*y(8)
    end subroutine rhs

end program channel
