! release.m
! Stochastic neurotransmitter release: ex12.m
! Ralf Schneggenburger & Erwin Neher
! Nature 406, 889-893, 2000}

program release
    use mtmod
    implicit none

    double precision, parameter :: ttime = 4.0d0   ! total time
    double precision, parameter :: dt    = 0.02d0  ! time step
    double precision, parameter :: peak  = 20.0d0  ! peak calcium concentration
    double precision, parameter :: hw    = 1.0d0   ! half width of calcium transient
    integer, parameter :: iter  = 100000           ! number of trials

! kinetic constants
    double precision, parameter :: kon    = 9.0d-2 ! microM^{-1} ms^{-1}
    double precision, parameter :: koff   = 9.5d0  ! ms^{-1}
    double precision, parameter :: beta   = 0.25d0 ! cooperativity
    double precision, parameter :: gamma  = 6.0d0  ! ms^{-1}
    double precision, parameter :: epsilon= 0.3d0  !  ms^{-1}
    integer, parameter :: initialState = 1

    double precision, parameter :: konh   = dt * kon;
    double precision, parameter :: koffh  = dt * koff;
    double precision, parameter :: gammah = dt * gamma;
    double precision, parameter :: epsilonh = dt*epsilon;
    double precision, parameter :: beta2  = beta*beta;
    double precision, parameter :: beta3  = beta*beta2;
    double precision, parameter :: beta4  = beta*beta3;

    integer, parameter :: npts = floor(ttime/dt)+1
    integer, dimension(npts) :: sum = 0
    double precision, dimension(npts) :: ca = 0.0d0
    integer :: i, k, counter, s, snext, seed
    double precision :: d, alpha, r, th

    write(*, fmt='(I10, " iterations")') iter
    
! ca transient
    d = 2.0d0*sqrt((1.0d0 - sqrt(0.5d0)))/hw
    k = int(2.0d0/(dt*d))
    do i=1,k
        ca(i) = peak * (((d*dt*(i-1) - 1.0d0)**2)-1.0)**2
    end do

    call system_clock(count=seed)
    call sgrnd(seed)

    counter = 0
    do k = 1,iter
        s  = initialState  ! initial state
        do i = 1,npts
            r = grnd()               ! get a random number [0, 1]
            alpha = konh * ca(i)     ! get Ca level
            snext = s

            select case (s)
                case (1)
                    th = 5.0 * alpha
                    if(r<th)then
                        snext = 2
                    end if
                case (2)
                    th = 4.0 * alpha
                    if(r < th)then
                        snext = 3
                    else
                        th = th + koffh
                        if(r<th)then
                            snext = 1
                        end if
                    end if
                case (3)
                    th = 3.0 * alpha
                    if(r < th)then
                        snext = 4
                    else
                        th = th + koffh * beta
                        if(r < th)then
                            snext = 2
                        end if
                    end if
                case (4)
                    th = 2.0 * alpha
                    if (r < th)then
                        snext = 5
                    else
                        th = th + koffh * beta2;
                        if(r < th)then
                            snext = 3
                        end if
                    end if
                case  (5)
                    th = alpha
                    if (r < th)then
                        snext = 6
                    else
                        th = th + koffh * beta3
                        if (r < th)then
                            snext = 4
                        end if
                    end if
                case (6)
                    th = gammah
                    if(r < th)then
                        snext = 7
                    else
                        th = th + koffh * beta4
                        if(r < th)then
                            snext = 5
                        end if
                    end if
                case (7 )     ! release
                    snext = 0           
                    sum(i) = sum(i) + 1
                    counter = counter + 1
                case (0)      !recovery from release
                    th = epsilonh
                    if(r < th)then
                        snext = 1
                    end if
            end select 
            s = snext
        end  do ! end of i-loop
    end do ! end of k-loop

    write(*, fmt='("release probability = ", F10.5)') dble(counter)/dble(iter)    


    open(unit = 11, file='ax.txt', form='formatted', access='stream', status='replace')
    do i = 1, npts
        write(unit=11, fmt='(3E15.7)') dt*i, ca(i), dble(sum(i))
    end do
    close(unit=11)
    call system("../../tools/gc ax.txt ");

end program release
