

public class Cell extends RungeKutta4 {

    final double Dm = 60.0;  // (micrometer) diameter of the cell
    final double Lm = 70.0;  // (micrometer) length of the cell

    final double Eleak = -65.0;   // (mV)
    final double ENa = 50.0;   // (mV)
    final double EK = -100.0;   // (mV)
    final double ECa = 50.0;   // (mV)

    final double dGleak = 0.002;// (mS/cm^2)
    final double dGNa   = 0.05; //0.53;   // (mS/cm^2)
    final double dGK    = 0.15; // 0.13;  // (mS/cm^2)
    final double dGTCa  = 0.03;  //  (mS/cm^2)

    final double vtraub = -55.0; //  -55.0 mV
    final double fac = 10;  // conversion factor (mS/cm^2) -> (pS/micrometer^2)

    int ndim;
//     double v,vinit;
    double Iinj;   // 2000.0 (pA)
    double Ton;    //   20 ms
    double Toff;   //   40 ms

    double Am;     //  surface area
    double Cm;
    double InvCm;  //  1/Cm;

    double Gleak;
    double GNa;
    double GK;
    double GTCa;
    double v_init;

    //    public Cell::setInitialValues(double v_init, vector<double>& y){
    public void setInitialValues(double vinit, double [] y){
        Am = 3.14 * Dm * Lm;    // (squre_micrometer) surface area
        Cm = 0.01 * Am;
        InvCm = 1.0/Cm;

        Gleak = Am * dGleak * fac;
        GNa   = Am * dGNa * fac;
        GK    = Am * dGK * fac;
        GTCa  = Am * dGTCa * fac;

        double v = y[0] = v_init = vinit;
        double [] aa = new double[2];

        get_Na_m(v, aa);
        y[1] = aa[0]/(aa[0]+aa[1]);

        get_Na_h(v,aa);
        y[2] = aa[0]/(aa[0]+aa[1]);

        get_K_n(v,aa);
        y[3] = aa[0]/(aa[0]+aa[1]);
        
        
        get_TCa_m(v,aa);
        y[4] = aa[1];

        get_TCa_h(v,aa);
        y[5] = aa[1]; 
    }

    public void setPulse(double ton, double toff, double current){
        Ton  = ton;
        Toff = toff;
        Iinj = current;
    }

    private void get_Na_m(double v, double [] aa){
        double v2 = v-vtraub;
        double a = 0.32*(13.0-v2)/(Math.exp((13.0-v2)/4.0)-1.0);
        double b = 0.28*(v2-40.0)/(Math.exp((v2-40.0)/5.0)-1.0);
        aa[0] = a;
        aa[1] = b;
    }

    private void get_Na_h(double v, double [] aa){
        double v2 = v-vtraub;
        double a = 0.128 * Math.exp((17.0-v2)/18.0);
        double b = 4.0/(1.0 + Math.exp((40.0-v2)/5.0));
        aa[0] = a;
        aa[1] = b;
    }

    private void get_K_n(double v, double [] aa){
        double v2 = v - vtraub;
        double a = 0.032*(15.0-v2)/(Math.exp((15.0-v2)/5.0)-1.0);
        double b = 0.5*Math.exp((10.0-v2)/40.0);
        aa[0] = a;
        aa[1] = b;
    }

    private void get_TCa_m(double v, double [] aa){
        double vshift=0.0;
        double vs = v + vshift;
        double tau = (3.0+1.0/(Math.exp((vs+25.0)/10.0)+Math.exp(-(vs+100.0)/15.0)));
        double inf = 1.0/(1.0+Math.exp(-(vs+50.0)/7.4));
        aa[0] = tau;
        aa[1] = inf;
    }

    private void get_TCa_h(double v, double [] aa){
        double vshift=0.0;
        double vs = v + vshift;
        double tau = (85.0+1.0/(Math.exp((vs+46.0)/4.0)+Math.exp(-(vs+405.0)/50.0))) ;
        double inf = 1.0/(1.0+Math.exp((vs+78.0)/5.0));
        aa[0] = tau;
        aa[1] = inf;
    }

    public void rhs(double tt, double [] y, double [] dy){
        double INa, IK, ITCa;
        double a, b, tau, inf;
        double shift = 0.0;
        double v = y[0];
        double c = (tt<Ton || tt>Toff)? 0.0 : Iinj;

        double [] aa = new double[2];

        INa = GNa * Math.pow(y[1],3) * y[2]  * (v-ENa); // INa = GNa * m^3 * h * (v-Ena)
        IK = GK * Math.pow(y[3],4) * (v-EK);            // IK = GK * n^4 (v-EK)
        ITCa = GTCa * Math.pow(y[4],2) * y[5] * (v - ECa);
        dy[0] = InvCm * (c - Gleak * (v-Eleak) - INa - IK - ITCa);
        // Na channel
        get_Na_m(v, aa);
        dy[1] = aa[0] - (aa[0]+aa[1])*y[1];

        get_Na_h(v, aa);
        dy[2] = aa[0] - (aa[0]+aa[1])*y[2];
        // K channel (delayed rectifier)
        get_K_n(v, aa);
        dy[3] = aa[0] - (aa[0]+aa[1])*y[3];
        // T-type Ca channel
        get_TCa_m(v, aa);
        dy[4] = (aa[1] - y[4]) / aa[0];
        get_TCa_h(v, aa);
        dy[5] = (aa[1] - y[5]) / aa[0];
    }
}
