// release.java
// Stochastic neurotransmitter release
// Ralf Schneggenburger & Erwin Neher
// Nature 406, 889-893, 2000.

import java.util.Random;
import java.io.FileWriter;
import java.io.IOException;

public class Release{

    public static void main(String [] args){
        double ttime = 4.0;
        double dt=0.02;
        double peak = 20.0;
        double hw = 1.0;
        int iter = 100000;

        double kon  =  9.0e-2; // microM^{-1} ms^{-1}
        double koff =  9.5;    // ms^{-1}
        double beta =  0.25;   // cooperativity
        double gamma = 6.0;    // 6.0d0  ! ms^{-1}
        double epsilon = 0.3;  // ms^{-1}
        int initialState = 1;

        System.out.println(iter + " iterations");

        int npts = (int)(ttime/dt);
        int counter = 0;
        double [] ca = new double[npts];
        int [] sum = new int[npts];

        java.util.Arrays.fill(ca,0.0);
        java.util.Arrays.fill(sum,0);
        
        Random ran = new Random();

        // ca transient
        double d = 2.0*Math.sqrt((1.0 - Math.sqrt(0.5)))/hw;
        for(int i=0;i<(int)(2.0/(dt*d));i++){
            ca[i] = peak * Math.pow(Math.pow(d*dt*i - 1.0, 2.0)-1.0, 2.0);
        }

        double konh   = dt * kon;
        double koffh  = dt * koff;
        double gammah = dt * gamma;
        double epsilonh = dt*epsilon;
        double beta2 = beta*beta;
        double beta3 = beta*beta2;
        double beta4 = beta*beta3;

        for(int k=0;k<iter;k++){    // iteration loop
            int s  = initialState;  //initial state
            for(int i=0;i<npts;i++){
                double th;
                double r = ran.nextDouble();                  // get a random number [0, 1]
                double alpha = konh * ca[i];  // get Ca level
                int snext = s;
                switch (s){
                    case (1):
                        th = 5.0 * alpha;
                        if(r<th) snext = 2;
                        break;
                    case (2):
                        th = 4.0 * alpha;
                        if(r<th) {
                            snext = 3;
                        }else{
                            th = th + koffh;
                            if(r<th) snext = 1;
                        }
                        break;
                    case (3):
                        th = 3.0 * alpha;
                        if(r<th) {
                            snext = 4;
                        }else{
                            th = th + koffh * beta;
                            if(r<th) snext = 2;
                        }
                        break;
                    case (4):
                        th = 2.0 * alpha;
                        if(r<th){
                            snext = 5;
                        }else{
                            th = th + koffh * beta2;
                            if(r<th) snext = 3;
                        }
                        break;
                    case (5):
                        th = alpha;
                        if(r<th){
                            snext = 6;
                        }else{
                            th = th + koffh * beta3;
                            if(r<th) snext = 4 ;
                        }
                        break;
                    case (6):
                        th = gammah;
                        if(r<th){
                            snext = 7;
                        }else{
                            th = th + koffh * beta4;
                            if(r<th) snext = 5;
                        }
                        break;
                    case (7):
                        snext = 0;     //release
                        sum[i] += 1;  // release
                        counter++;
                        break;
                    case (0):         //recovery from release
                        th = epsilonh;
                        if(r<th) snext = 1;
                        break;
                    default:
                        ;
                } // end of switch
                s = snext;
            } //end of i-loop
        } // end of k-loop

        System.out.println("release probability = " + (double)counter/(double)iter);

        try{
            FileWriter fw = new FileWriter("ax.txt");
            for(int i=0;i<npts;i++){
                fw.write(dt*i+" "+ ca[i] +" "+ (double)sum[i]+"\n");
            }
            fw.close();
        }catch(IOException e){
            System.out.println(e);
        }



        String cmd;
        String OS = System.getProperty("os.name", "generic").toLowerCase();
        if (OS.indexOf("win") >= 0) {
            cmd = "cmd /c ..\\..\\tools\\gc ax.txt";            
        }else{ // mac
            cmd = "..//..//tools//gc ax.txt";
        }
        
        try{
            Process p = Runtime.getRuntime().exec(cmd);
            p.waitFor();
        } catch (Exception e) {}
    }
}