"""
release.py
Stochastic neurotransmitter release
Ralf Schneggenburger & Erwin Neher
Nature 406, 889-893, 2000
"""
import numpy as np
import random
import math
import matplotlib.pyplot as plt

def release():
    ttime = 4.0
    dt    = 0.02
    peak  = 20.0
    hw    = 1.0
    iter  = 100000

    npts = (int)(ttime/dt)+1
    tt  = np.linspace(0.0, ttime, npts)
    sum = np.zeros(npts, int)
    ca  = np.zeros(npts, float)

    print('{0:d} iterations'.format(iter))

    # ca transient
    d = 2.0*math.sqrt((1.0 - math.sqrt(0.5)))/hw
    k = (int)(2.0/(dt*d))
    ca[0:k] = peak * pow(pow(d*tt[0:k] - 1.0, 2.0)-1.0, 2.0)

    kon   =  9.0e-2 # microM^{-1} ms^{-1}
    koff  =  9.5    # ms^{-1}
    beta  =  0.25   # cooperativity
    gamma = 6.0     # 6.0d0  ! ms^{-1}
    epsilon = 0.3   # ms^{-1}
    initialState = 1

    konh   = dt * kon
    koffh  = dt * koff
    gammah = dt * gamma
    epsilonh = dt*epsilon
    beta2  = beta*beta
    beta3  = beta*beta2
    beta4  = beta*beta3

    counter = 0
    random.seed()

    for k in range(iter):
        s  = initialState  #initial state
        for i in range(npts):
            r = random.random()   # get a random number [0, 1]
            alpha = konh * ca[i]  # get Ca level
            snext = s
            if s == 1:
                th = 5.0 * alpha
                if r < th:
                    snext = 2
            elif s == 2:
                th = 4.0 * alpha
                if r < th:
                    snext = 3
                else:
                    th = th + koffh
                    if r < th:
                        snext = 1
            elif s == 3:
                th = 3.0 * alpha
                if r < th:
                    snext = 4
                else:
                    th = th + koffh * beta
                    if r < th:
                        snext = 2
            elif s == 4:
                th = 2.0 * alpha
                if r < th:
                    snext = 5
                else:
                    th = th + koffh * beta2
                    if r < th:
                        snext = 3
            elif s == 5:
                th = alpha
                if r < th:
                    snext = 6
                else:
                    th = th + koffh * beta3
                    if r < th:
                        snext = 4
            elif s == 6:
                th = gammah
                if r < th:
                    snext = 7
                else:
                    th = th + koffh * beta4
                    if r < th:
                        snext = 5
            elif s == 7:
                snext = 0     #release
                sum[i] += 1  # release
                counter += 1
            elif s == 0:         #recovery from release
                th = epsilonh
                if r < th:
                    snext = 1

            s = snext;

    print('release probability = {0:.4f}'.format(float(counter)/iter))

#    plt.figure()
#    plt.plot(tt,sum, tt, ca)
#    plt.show()

if __name__ == '__main__':
    release()
